@@ -8,23 +8,31 @@
     :license: BSD, see LICENSE for more details.
 """
 
+from __future__ import annotations
+
 import logging
 import math
 import operator
 import re
 from collections.abc import Mapping
-from decimal import Decimal
 from fractions import Fraction
-from functools import lru_cache
+from functools import lru_cache, partial
 from logging import NullHandler
 from numbers import Number
 from token import NAME, NUMBER
+from typing import TYPE_CHECKING, ClassVar, Optional, Union
 
 from .compat import NUMERIC_TYPES, tokenizer
 from .errors import DefinitionSyntaxError
 from .formatting import format_unit
 from .pint_eval import build_eval_tree
 
+if TYPE_CHECKING:
+    from ._typing import UnitLike
+    from .quantity import Quantity
+    from .registry import BaseRegistry
+
+
 logger = logging.getLogger(__name__)
 logger.addHandler(NullHandler())
 
@@ -178,14 +186,16 @@ def pi_theorem(quantities, registry=None):
 
     if registry is None:
         getdim = lambda x: x
+        non_int_type = float
     else:
         getdim = registry.get_dimensionality
+        non_int_type = registry.non_int_type
 
     for name, value in quantities.items():
         if isinstance(value, str):
-            value = ParserHelper.from_string(value)
+            value = ParserHelper.from_string(value, non_int_type=non_int_type)
         if isinstance(value, dict):
-            dims = getdim(UnitsContainer(value))
+            dims = getdim(registry.UnitsContainer(value))
         elif not hasattr(value, "dimensionality"):
             dims = getdim(value)
         else:
@@ -296,7 +306,7 @@ class udict(dict):
     """Custom dict implementing __missing__."""
 
     def __missing__(self, key):
-        return 0.0
+        return 0
 
     def copy(self):
         return udict(self)
@@ -318,9 +328,21 @@ class UnitsContainer(Mapping):
 
     """
 
-    __slots__ = ("_d", "_hash")
+    __slots__ = ("_d", "_hash", "_one", "_non_int_type")
+
+    def __init__(self, *args, **kwargs) -> None:
+        if args and isinstance(args[0], UnitsContainer):
+            default_non_int_type = args[0]._non_int_type
+        else:
+            default_non_int_type = float
+
+        self._non_int_type = kwargs.pop("non_int_type", default_non_int_type)
+
+        if self._non_int_type is float:
+            self._one = 1
+        else:
+            self._one = self._non_int_type("1")
 
-    def __init__(self, *args, **kwargs):
         d = udict(*args, **kwargs)
         self._d = d
         for key, value in d.items():
@@ -328,8 +350,8 @@ class UnitsContainer(Mapping):
                 raise TypeError("key must be a str, not {}".format(type(key)))
             if not isinstance(value, Number):
                 raise TypeError("value must be a number, not {}".format(type(value)))
-            if not isinstance(value, float):
-                d[key] = float(value)
+            if not isinstance(value, int) and not isinstance(value, self._non_int_type):
+                d[key] = self._non_int_type(value)
         self._hash = None
 
     def copy(self):
@@ -385,7 +407,7 @@ class UnitsContainer(Mapping):
     def __iter__(self):
         return iter(self._d)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self._d)
 
     def __getitem__(self, key):
@@ -399,7 +421,14 @@ class UnitsContainer(Mapping):
             self._hash = hash(frozenset(self._d.items()))
         return self._hash
 
-    def __eq__(self, other):
+    # Only needed by pickle protocol 0 and 1 (used by pytables)
+    def __getstate__(self):
+        return self._d, self._hash, self._one, self._non_int_type
+
+    def __setstate__(self, state):
+        self._d, self._hash, self._one, self._non_int_type = state
+
+    def __eq__(self, other) -> bool:
         if isinstance(other, UnitsContainer):
             # UnitsContainer.__hash__(self) is not the same as hash(self); see
             # ParserHelper.__hash__ and __eq__.
@@ -411,31 +440,37 @@ class UnitsContainer(Mapping):
             other = other._d
 
         elif isinstance(other, str):
-            other = ParserHelper.from_string(other)
+            try:
+                other = ParserHelper.from_string(other, self._non_int_type)
+            except DefinitionSyntaxError:
+                return False
+
             other = other._d
 
         return dict.__eq__(self._d, other)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.__format__("")
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         tmp = "{%s}" % ", ".join(
             ["'{}': {}".format(key, value) for key, value in sorted(self._d.items())]
         )
         return "<UnitsContainer({})>".format(tmp)
 
-    def __format__(self, spec):
+    def __format__(self, spec: str) -> str:
         return format_unit(self, spec)
 
-    def format_babel(self, spec, **kwspec):
-        return format_unit(self, spec, **kwspec)
+    def format_babel(self, spec: str, registry=None, **kwspec) -> str:
+        return format_unit(self, spec, registry=registry, **kwspec)
 
     def __copy__(self):
         # Skip expensive health checks performed by __init__
         out = object.__new__(self.__class__)
         out._d = self._d.copy()
         out._hash = self._hash
+        out._non_int_type = self._non_int_type
+        out._one = self._one
         return out
 
     def __mul__(self, other):
@@ -512,7 +547,7 @@ class ParserHelper(UnitsContainer):
         self.scale = scale
 
     @classmethod
-    def from_word(cls, input_word):
+    def from_word(cls, input_word, non_int_type=float):
         """Creates a ParserHelper object with a single variable with exponent one.
 
         Equivalent to: ParserHelper({'word': 1})
@@ -526,27 +561,41 @@ class ParserHelper(UnitsContainer):
         -------
 
         """
-        return cls(1, [(input_word, 1)])
+        if non_int_type is float:
+            return cls(1, [(input_word, 1)], non_int_type=non_int_type)
+        else:
+            ONE = non_int_type("1.0")
+            return cls(ONE, [(input_word, ONE)], non_int_type=non_int_type)
 
     @classmethod
-    def eval_token(cls, token, use_decimal=False):
+    def eval_token(cls, token, use_decimal=False, non_int_type=float):
+
+        # TODO: remove this code when use_decimal is deprecated
+        if use_decimal:
+            raise DeprecationWarning(
+                "`use_decimal` is deprecated, use `non_int_type` keyword argument when instantiating the registry.\n"
+                ">>> from decimal import Decimal\n"
+                ">>> ureg = UnitRegistry(non_int_type=Decimal)"
+            )
+
         token_type = token.type
         token_text = token.string
         if token_type == NUMBER:
-            try:
-                return int(token_text)
-            except ValueError:
-                if use_decimal:
-                    return Decimal(token_text)
-                return float(token_text)
+            if non_int_type is float:
+                try:
+                    return int(token_text)
+                except ValueError:
+                    return float(token_text)
+            else:
+                return non_int_type(token_text)
         elif token_type == NAME:
-            return ParserHelper.from_word(token_text)
+            return ParserHelper.from_word(token_text, non_int_type=non_int_type)
         else:
             raise Exception("unknown token type")
 
     @classmethod
     @lru_cache()
-    def from_string(cls, input_string):
+    def from_string(cls, input_string, non_int_type=float):
         """Parse linear expression mathematical units and return a quantity object.
 
         Parameters
@@ -559,7 +608,7 @@ class ParserHelper(UnitsContainer):
 
         """
         if not input_string:
-            return cls()
+            return cls(non_int_type=non_int_type)
 
         input_string = string_preprocessor(input_string)
         if "[" in input_string:
@@ -571,10 +620,12 @@ class ParserHelper(UnitsContainer):
             reps = False
 
         gen = tokenizer(input_string)
-        ret = build_eval_tree(gen).evaluate(cls.eval_token)
+        ret = build_eval_tree(gen).evaluate(
+            partial(cls.eval_token, non_int_type=non_int_type)
+        )
 
         if isinstance(ret, Number):
-            return ParserHelper(ret)
+            return ParserHelper(ret, non_int_type=non_int_type)
 
         if reps:
             ret = ParserHelper(
@@ -583,6 +634,7 @@ class ParserHelper(UnitsContainer):
                     key.replace("__obra__", "[").replace("__cbra__", "]"): value
                     for key, value in ret.items()
                 },
+                non_int_type=non_int_type,
             )
 
         for k in list(ret):
@@ -601,20 +653,28 @@ class ParserHelper(UnitsContainer):
         return self.__copy__()
 
     def __hash__(self):
-        if self.scale != 1.0:
-            mess = "Only scale 1.0 ParserHelper instance should be considered hashable"
+        if self.scale != 1:
+            mess = "Only scale 1 ParserHelper instance should be considered hashable"
             raise ValueError(mess)
         return super().__hash__()
 
+    # Only needed by pickle protocol 0 and 1 (used by pytables)
+    def __getstate__(self):
+        return super().__getstate__() + (self.scale,)
+
+    def __setstate__(self, state):
+        super().__setstate__(state[:-1])
+        self.scale = state[-1]
+
     def __eq__(self, other):
         if isinstance(other, ParserHelper):
             return self.scale == other.scale and super().__eq__(other)
         elif isinstance(other, str):
-            return self == ParserHelper.from_string(other)
+            return self == ParserHelper.from_string(other, self._non_int_type)
         elif isinstance(other, Number):
             return self.scale == other and not len(self._d)
         else:
-            return self.scale == 1.0 and super().__eq__(other)
+            return self.scale == 1 and super().__eq__(other)
 
     def operate(self, items, op=operator.iadd, cleanup=True):
         d = udict(self._d)
@@ -626,7 +686,7 @@ class ParserHelper(UnitsContainer):
             for key in keys:
                 del d[key]
 
-        return self.__class__(self.scale, d)
+        return self.__class__(self.scale, d, non_int_type=self._non_int_type)
 
     def __str__(self):
         tmp = "{%s}" % ", ".join(
@@ -642,7 +702,7 @@ class ParserHelper(UnitsContainer):
 
     def __mul__(self, other):
         if isinstance(other, str):
-            new = self.add(other, 1)
+            new = self.add(other, self._one)
         elif isinstance(other, Number):
             new = self.copy()
             new.scale *= other
@@ -659,7 +719,7 @@ class ParserHelper(UnitsContainer):
         d = self._d.copy()
         for key in self._d:
             d[key] *= other
-        return self.__class__(self.scale ** other, d)
+        return self.__class__(self.scale ** other, d, non_int_type=self._non_int_type)
 
     def __truediv__(self, other):
         if isinstance(other, str):
@@ -679,7 +739,7 @@ class ParserHelper(UnitsContainer):
     def __rtruediv__(self, other):
         new = self.__pow__(-1)
         if isinstance(other, str):
-            new = new.add(other, 1)
+            new = new.add(other, self._one)
         elif isinstance(other, Number):
             new.scale *= other
         elif isinstance(other, self.__class__):
@@ -691,7 +751,7 @@ class ParserHelper(UnitsContainer):
 
 
 #: List of regex substitution pairs.
-_subs_re = [
+_subs_re_list = [
     ("\N{DEGREE SIGN}", " degree"),
     (r"([\w\.\-\+\*\\\^])\s+", r"\1 "),  # merge multiple spaces
     (r"({}) squared", r"\1**2"),  # Handle square and cube
@@ -707,22 +767,22 @@ _subs_re = [
 ]
 
 #: Compiles the regex and replace {} by a regex that matches an identifier.
-_subs_re = [(re.compile(a.format(r"[_a-zA-Z][_a-zA-Z0-9]*")), b) for a, b in _subs_re]
+_subs_re = [
+    (re.compile(a.format(r"[_a-zA-Z][_a-zA-Z0-9]*")), b) for a, b in _subs_re_list
+]
 _pretty_table = str.maketrans("⁰¹²³⁴⁵⁶⁷⁸⁹·⁻", "0123456789*-")
-_pretty_exp_re = re.compile(r"⁻?[⁰¹²³⁴⁵⁶⁷⁸⁹]+(?:\.[⁰¹²³⁴⁵⁶⁷⁸⁹]*)?")
+_pretty_exp_re = re.compile(r"(⁻?[⁰¹²³⁴⁵⁶⁷⁸⁹]+(?:\.[⁰¹²³⁴⁵⁶⁷⁸⁹]*)?)")
 
 
-def string_preprocessor(input_string):
+def string_preprocessor(input_string: str) -> str:
     input_string = input_string.replace(",", "")
     input_string = input_string.replace(" per ", "/")
 
     for a, b in _subs_re:
         input_string = a.sub(b, input_string)
 
+    input_string = _pretty_exp_re.sub(r"**(\1)", input_string)
     # Replace pretty format characters
-    for pretty_exp in _pretty_exp_re.findall(input_string):
-        exp = "**" + pretty_exp.translate(_pretty_table)
-        input_string = input_string.replace(pretty_exp, exp)
     input_string = input_string.translate(_pretty_table)
 
     # Handle caret exponentiation
@@ -730,7 +790,7 @@ def string_preprocessor(input_string):
     return input_string
 
 
-def _is_dim(name):
+def _is_dim(name: str) -> bool:
     return name[0] == "[" and name[-1] == "]"
 
 
@@ -748,17 +808,20 @@ class SharedRegistryObject:
 
     """
 
+    _REGISTRY: ClassVar[BaseRegistry]
+    _units: UnitsContainer
+
     def __new__(cls, *args, **kwargs):
         inst = object.__new__(cls)
         if not hasattr(cls, "_REGISTRY"):
             # Base class, not subclasses dynamically by
             # UnitRegistry._init_dynamic_classes
-            from . import _APP_REGISTRY
+            from . import application_registry
 
-            inst._REGISTRY = _APP_REGISTRY
+            inst._REGISTRY = application_registry.get()
         return inst
 
-    def _check(self, other):
+    def _check(self, other) -> bool:
         """Check if the other object use a registry and if so that it is the
         same registry.
 
@@ -789,6 +852,8 @@ class SharedRegistryObject:
 class PrettyIPython:
     """Mixin to add pretty-printers for IPython"""
 
+    default_format: str
+
     def _repr_html_(self):
         if "~" in self.default_format:
             return "{:~H}".format(self)
@@ -808,7 +873,9 @@ class PrettyIPython:
             p.text("{:P}".format(self))
 
 
-def to_units_container(unit_like, registry=None):
+def to_units_container(
+    unit_like: Union[UnitLike, Quantity], registry: Optional[BaseRegistry] = None
+) -> UnitsContainer:
     """Convert a unit compatible type to a UnitsContainer.
 
     Parameters
@@ -833,7 +900,10 @@ def to_units_container(unit_like, registry=None):
         else:
             return ParserHelper.from_string(unit_like)
     elif dict in mro:
-        return UnitsContainer(unit_like)
+        if registry:
+            return registry.UnitsContainer(unit_like)
+        else:
+            return UnitsContainer(unit_like)
 
 
 def infer_base_unit(q):
@@ -862,12 +932,16 @@ def infer_base_unit(q):
 
 
 def getattr_maybe_raise(self, item):
-    """Helper function to invoke at the beginning of all overridden ``__getattr__``
-    methods. Raise AttributeError if the user tries to ask for a _ or __ attribute.
+    """Helper function invoked at start of all overridden ``__getattr__``.
+
+    Raise AttributeError if the user tries to ask for a _ or __ attribute,
+    *unless* it is immediately followed by a number, to enable units
+    encompassing constants, such as ``L / _100km``.
 
     Parameters
     ----------
-    item :
+    item : string
+        Item to be found.
 
 
     Returns
@@ -876,7 +950,11 @@ def getattr_maybe_raise(self, item):
     """
     # Double-underscore attributes are tricky to detect because they are
     # automatically prefixed with the class name - which may be a subclass of self
-    if item.startswith("_") or item.endswith("__"):
+    if (
+        item.endswith("__")
+        or len(item.lstrip("_")) == 0
+        or (item.startswith("_") and not item.lstrip("_")[0].isdigit())
+    ):
         raise AttributeError("%r object has no attribute %r" % (self, item))
 
 
@@ -952,7 +1030,7 @@ class BlockIterator(SourceIterator):
     next = __next__
 
 
-def iterable(y):
+def iterable(y) -> bool:
     """Check whether or not an object can be iterated over.
 
     Vendored from numpy under the terms of the BSD 3-Clause License. (Copyright
@@ -974,7 +1052,7 @@ def iterable(y):
     return True
 
 
-def sized(y):
+def sized(y) -> bool:
     """Check whether or not an object has a defined length.
 
     Parameters
